/**
* @author H. Timucin OZDEMIR
*/

import java.applet.*;
import java.awt.*;
import java.net.*;
import java.io.*;
import java.util.*;

import org.omg.CosNaming.*;
import org.omg.CORBA.*;

public class 
GrBenchClient extends Applet implements Runnable
{
    Grapher panel;
    TestInputPanel inputPanel;
    Thread runner;
    int type = 0;
    int t_size = 100;
    int arr_size = 100;
    int step = 50;
    Benchmarker sampler;
    Benchmark   benchInstance;

    public void 
    init() 
    {
        setLayout(new BorderLayout());

        panel = new Grapher(100);
        add("Center", panel);

        inputPanel = new TestInputPanel(this);
        add("West", inputPanel);

        sampler = new Benchmarker();
        getServer(); 

    }// end of init()

    /**
    */
    String getIOR()
    {

       String retIOR=null;


       try
       {
          URL address = getCodeBase();
          String host = address.getHost();
          int port = address.getPort();

          String query = new String("http://"+host+":"+port+"/servlet/naming?");
          query += "cmd=getIOR&";
          query += "IDL=IDL:Benchmark:1.0";
          System.out.println(" query :"+query);

          URL namingURL = new URL(query);

          DataInputStream dis = new DataInputStream( namingURL.openStream());
          retIOR = dis.readLine();

       } catch(MalformedURLException mfue) {}
         catch(IOException ioe) {}

       return retIOR;

    }// end of getIOR()


    /**
    */
    void getServer()
    {
       String ior = getIOR();

       if(ior!=null)
       {

          Properties props = new Properties();
          props.put("org.omg.CORBA.ORBClass", "com.sun.CORBA.iiop.ORB");
          props.put("org.omg.CORBA.ORBSingletonClass", "com.sun.CORBA.iiop.ORBSingleton");
          props.put("ORBInitialHost", getParameter("ORBInitialHost"));
          props.put("ORBInitialPort", getParameter("ORBInitialPort"));


          System.out.println(" org.omg.CORBA.ORBClass : "+"com.sun.CORBA.iiop.ORB");
          System.out.println(" ORBInitialHost : "+getParameter("ORBInitialHost"));
          System.out.println(" ORBInitialPort : "+getParameter("ORBInitialPort"));
          ORB orb = ORB.init(this,props);

          org.omg.CORBA.Object objRef = orb.string_to_object(ior);
          benchInstance = BenchmarkHelper.narrow(objRef);

       }// end of if

    }// end of getServer() 

    /**
    */
    void doTestPing(int NumOfIterations)
    {
       Stat ret;
       int x=0;
       while(x<NumOfIterations)
       {
          ret = sampler.doPingFor(benchInstance,step);
          panel.addPoint(ret.average);
          x += step;

       }// end of while
       System.out.println(" doTestPing() is over! ");

    }// end of doTestPing()

    /**
    */
    void doTestIntArray(int NumOfIterations, int arraySize)
    {
       Stat ret;
       int x=0;

       while(x<NumOfIterations)
       {
          ret = sampler.doIntArrayFor(benchInstance,step,arraySize);
          panel.addPoint(ret.average);
          x += step;

       }// end of while
       System.out.println(" doTestIntArray() is over! ");

    }// end of doTestIntArray()

    /**
    */
    void doTestStructArray(int NumOfIterations, int arraySize)
    {
       Stat ret;
       int x=0;
       while(x<NumOfIterations)
       {
          ret = sampler.doStructArrayFor(benchInstance,step,arraySize);
          panel.addPoint(ret.average);
          x += step;

       }// end of while
       System.out.println(" doTestStructArray() is over! ");

    }// end of doTestIntArray()

    /**
    */
    public void 
    startThread() 
    {
      if(runner==null)
      {
         runner=new Thread(this);
         runner.setPriority(Thread.MIN_PRIORITY);
         runner.start();
      } 
      else 
        { System.out.println(" runner thread is *NOT NULL*  ");
          runner.start();
        }

    }

    /**
    */
    double 
    f(double x)
    {
       return (Math.cos(x/5) + Math.sin(x/7) + 2) * size().height / 4;
    }

    void
    drawF()
    {
        int x= 0;
        while(true)
        {

           //panel.addPoint((float)f((x%4)*0.01333f));
           panel.addPoint(((x+1) % 101));
           x = (x+1) % 100; 
           //System.out.println("x "+x);
        } 
    }


    /**
    */
    public void 
    run() 
    {
        //panel.start();

        //drawF();

        while(true)
        {
           switch(type)
           {
              case 1 : doTestIntArray(t_size,arr_size); break;
              case 2 : doTestStructArray(t_size,arr_size); break;

              default: doTestPing(t_size); break;

           }// end of switch()

        }// end of while
        // 

    }

    /**
    */
    public void 
    stop() 
    {
        System.out.println(" Thread is stopped!");
        if(runner!=null) runner.stop();
        runner = null;
        if(runner==null) System.out.println(" runner is NULL!");
    }

    /**
    */
    public boolean 
    action(Event evt, java.lang.Object arg) 
    {
        return false;
    }

    /**
    */
    public String 
    getAppletInfo() 
    {
      return "Title: Benchmarker \nAuthor: H. Timucin Ozdemir";
    }
    
    /**
    *
    * 1. Stop the current benchmark
    * 2. Update the array size
    * 3. Resume the benchmark.
    *
    * Note: Do not allow to start a benchmark before completing the
    *       necessary settings.
    */
    public void
    setArraySize(int arrSize)
    {

      if(arrSize>0) arr_size = arrSize;
     
    }// end of setArraySize()

    public void
    setTestType(int _type)
    {
       if(_type>=0 && _type<4) type = _type;
    }

    public void
    setTestSize(int _size)
    {
       if(_size>0) t_size = _size;
    }

}// end of Class Bench
